package com.link.core.converter;

import com.link.core.converter.i18n.DefaultStringValue;
import com.link.core.converter.i18n.Resources;
import com.link.core.converter.util.RegularCloneable;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class HashMapConverter<SK, SV, TK, TV> implements Converter<Object, HashMap<TK, TV>>, HasConverters, RegularCloneable {
   private static final boolean DEFAULT_OPTIMAL = true;
   private Converter<SK, TK> keyConverter;
   private Converter<SV, TV> valueConverter;
   private boolean optimal;

   public static <K, V> HashMapConverter<K, V, K, V> newConverter(Class<K> keyType, Class<V> valueType) {
      return new HashMapConverter((Converter)null, (Converter)null);
   }

   public static <SK, SV, TK, TV> HashMapConverter<SK, SV, TK, TV> newConverter(Class<SK> sourceKeyType, Class<SV> sourceValueType, Class<TK> targetKeyType, Class<TV> targetValueType) {
      return new HashMapConverter((Converter)null, (Converter)null);
   }

   public static <K, SV, TV> HashMapConverter<K, SV, K, TV> newConverter(Class<K> keyType, Converter<SV, TV> valueConverter) {
      return new HashMapConverter((Converter)null, valueConverter);
   }

   public static <SK, TK, V> HashMapConverter<SK, V, TK, V> newConverter(Converter<SK, TK> keyConverter, Class<V> valueType) {
      return new HashMapConverter(keyConverter, (Converter)null);
   }

   public static <SK, SV, TK, TV> HashMapConverter<SK, SV, TK, TV> newConverter(Converter<SK, TK> keyConverter, Class<SV> sourceValueType, Class<TV> targetValueType) {
      return new HashMapConverter(keyConverter, (Converter)null);
   }

   public static <SK, SV, TK, TV> HashMapConverter<SK, SV, TK, TV> newConverter(Class<SK> sourceKeyType, Class<TK> targetKeyType, Converter<SV, TV> valueConverter) {
      return new HashMapConverter((Converter)null, valueConverter);
   }

   public static <SK, SV, TK, TV> HashMapConverter<SK, SV, TK, TV> newConverter(Converter<SK, TK> keyConverter, Converter<SV, TV> valueConverter) {
      return new HashMapConverter(keyConverter, valueConverter);
   }

   protected HashMapConverter() {
      this((Converter)null, (Converter)null);
   }

   protected HashMapConverter(Converter<SK, TK> keyConverter, Converter<SV, TV> valueConverter) {
      this.optimal = true;
      this.keyConverter = keyConverter;
      this.valueConverter = valueConverter;
   }

   public Converter<SK, TK> getKeyConverter() {
      return this.keyConverter;
   }

   public HashMapConverter<SK, SV, TK, TV> setKeyConverter(Converter<SK, TK> keyConverter) {
      this.keyConverter = keyConverter;
      return this;
   }

   public Converter<SV, TV> getValueConverter() {
      return this.valueConverter;
   }

   public HashMapConverter<SK, SV, TK, TV> setValueConverter(Converter<SV, TV> valueConverter) {
      this.valueConverter = valueConverter;
      return this;
   }

   public boolean isOptimal() {
      return this.optimal;
   }

   public HashMapConverter<SK, SV, TK, TV> setOptimal(boolean optimal) {
      this.optimal = optimal;
      return this;
   }

   public List<Converter> getConverters() {
      List<Converter> list = new ArrayList();
      if (this.keyConverter != null) {
         list.add(this.keyConverter);
      }

      if (this.valueConverter != null) {
         list.add(this.valueConverter);
      }

      return list;
   }

   public HashMap<TK, TV> convert(Object source) throws ConversionException {
      if (source == null) {
         return null;
      } else {
         try {
            if (source instanceof Map.Entry[]) {
               return this.convertFromArray((Map.Entry[])((Map.Entry[])source));
            } else if (source instanceof Iterable) {
               return this.convertFromIterable((Iterable)source);
            } else if (source instanceof Map) {
               return this.convertFromMap((Map)source);
            } else {
               throw new IllegalArgumentException(MessageFormat.format(HashMapConverter.R.R.unsupportedSourceClass(), source.getClass().getName()));
            }
         } catch (ClassCastException var3) {
            ClassCastException e = var3;
            throw new IllegalArgumentException(MessageFormat.format(HashMapConverter.R.R.unsupportedSourceClass(), source.getClass().getName()), e);
         }
      }
   }

   public Object clone() {
      HashMapConverter<SK, SV, TK, TV> other = new HashMapConverter();
      other.copyFrom(this);
      return other;
   }

   public void copyFrom(Object source) {
      if (source != null && source instanceof HashMapConverter) {
         HashMapConverter<SK, SV, TK, TV> sc = (HashMapConverter)source;
         if (sc.keyConverter instanceof RegularCloneable) {
            this.keyConverter = (Converter)((RegularCloneable)sc.keyConverter).clone();
         } else {
            this.keyConverter = sc.keyConverter;
         }

         if (sc.valueConverter instanceof RegularCloneable) {
            this.valueConverter = (Converter)((RegularCloneable)sc.valueConverter).clone();
         } else {
            this.valueConverter = sc.valueConverter;
         }

         this.optimal = sc.optimal;
      }
   }

   private HashMap<TK, TV> convertFromArray(Map.Entry<SK, SV>[] source) throws ConversionException {
      assert source != null;

      HashMap<TK, TV> target = new HashMap();
      Map<SK, TK> keyBuffer = new HashMap();
      Map<SV, TV> valueBuffer = new HashMap();
      Map.Entry[] var5 = source;
      int var6 = source.length;

      for(int var7 = 0; var7 < var6; ++var7) {
         Map.Entry<SK, SV> sourceItem = var5[var7];
         if (sourceItem != null) {
            TK targetKey = this.convertKey(sourceItem.getKey(), keyBuffer);
            TV targetValue = this.convertValue(sourceItem.getValue(), valueBuffer);
            target.put(targetKey, targetValue);
         }
      }

      return target;
   }

   private HashMap<TK, TV> convertFromIterable(Iterable<Map.Entry<SK, SV>> source) throws ConversionException {
      assert source != null;

      HashMap<TK, TV> target = new HashMap();
      Map<SK, TK> keyBuffer = new HashMap();
      Map<SV, TV> valueBuffer = new HashMap();
      Iterator var5 = source.iterator();

      while(var5.hasNext()) {
         Map.Entry<SK, SV> sourceItem = (Map.Entry)var5.next();
         if (sourceItem != null) {
            TK targetKey = this.convertKey(sourceItem.getKey(), keyBuffer);
            TV targetValue = this.convertValue(sourceItem.getValue(), valueBuffer);
            target.put(targetKey, targetValue);
         }
      }

      return target;
   }

   private HashMap<TK, TV> convertFromMap(Map<SK, SV> source) throws ConversionException {
      assert source != null;

      HashMap<TK, TV> target = new HashMap();
      Map<SK, TK> keyBuffer = new HashMap();
      Map<SV, TV> valueBuffer = new HashMap();
      Iterator var5 = source.entrySet().iterator();

      while(var5.hasNext()) {
         Map.Entry<SK, SV> sourceEntry = (Map.Entry)var5.next();
         TK targetKey = this.convertKey(sourceEntry.getKey(), keyBuffer);
         TV targetValue = this.convertValue(sourceEntry.getValue(), valueBuffer);
         target.put(targetKey, targetValue);
      }

      return target;
   }

   private TK convertKey(SK source, Map<SK, TK> buffer) throws ConversionException {
      if (this.keyConverter == null) {
         return source;
      } else if (!this.optimal) {
         return this.keyConverter.convert(source);
      } else {
         assert buffer != null;

         if (buffer.containsKey(source)) {
            return buffer.get(source);
         } else {
            TK target = this.keyConverter.convert(source);
            buffer.put(source, target);
            return target;
         }
      }
   }

   private TV convertValue(SV source, Map<SV, TV> buffer) throws ConversionException {
      if (this.valueConverter == null) {
         return source;
      } else if (!this.optimal) {
         return this.valueConverter.convert(source);
      } else {
         assert buffer != null;

         if (buffer.containsKey(source)) {
            return buffer.get(source);
         } else {
            TV target = this.valueConverter.convert(source);
            buffer.put(source, target);
            return target;
         }
      }
   }

   public interface R {
      R R = (R)Resources.create(R.class);

      @DefaultStringValue("不支持source对象类型“{0}”。")
      String unsupportedSourceClass();
   }
}
